#
# Copyright (c) 1995 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Computer Systems
#	Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: sack.tcl,v 1.3 95/11/29 23:29:14 floyd Exp $
#
# Create a simple four node topology:
#
#        s1
#         \ 
#  8Mb,5ms \  0.8Mb,100ms
#           r1 --------- k1
#  8Mb,5ms /
#         /
#        s2
#
proc create_testnet { } {

	global s1 s2 r1 k1
	global ns_tcp
	set s1 [ns node]
	set s2 [ns node]
	set r1 [ns node]
	set k1 [ns node]

	ns_duplex $s1 $r1 8Mb 5ms drop-tail
	ns_duplex $s2 $r1 8Mb 5ms drop-tail
	set L [ns_duplex $r1 $k1 800Kb 100ms drop-tail]
	[lindex $L 0] set queue-limit 6
	[lindex $L 1] set queue-limit 6
	# maxburst must be set to an integer.

	set ns_tcp(maxburst) maxBurst
	set ns_tcp(bug-fix) false
}

#
# Create a simple six node topology:
#
#        s1		    s3
#         \ 		    /
# 10Mb,2ms \  1.5Mb,20ms   / 10Mb,4ms
#           r1 --------- r2
# 10Mb,3ms /		   \ 10Mb,5ms
#         /		    \
#        s2		    s4 
#
proc create_testnet2 { } {

	global s1 s2 r1 r2 s3 s4
	global ns_tcp
	set s1 [ns node]
	set s2 [ns node]
	set r1 [ns node]
	set r2 [ns node]
	set s3 [ns node]
	set s4 [ns node]

	ns_duplex $s1 $r1 10Mb 2ms drop-tail
	ns_duplex $s2 $r1 10Mb 3ms drop-tail
	set L [ns_duplex $r1 $r2 1.5Mb 20ms red]
	[lindex $L 0] set queue-limit 25
	[lindex $L 1] set queue-limit 25
	ns_duplex $s3 $r2 10Mb 4ms drop-tail
	ns_duplex $s4 $r2 10Mb 5ms drop-tail
	set ns_tcp(maxburst) maxBurst
	set ns_tcp(bug-fix) false
}

proc finish file {

	set f [open temp.rands w]
	puts $f "TitleText: $file"
	puts $f "Device: Postscript"
	
	exec rm -f temp.p 
	exec rm -f temp.d 
	exec touch temp.d temp.p
	#
	# split queue/drop events into two separate files.
	# we don't bother checking for the link we're interested in
	# since we know only such events are in our trace file
	#
	exec awk {
		{
			if (($1 == "+" || $1 == "-" ) && \
			    ($5 == "tcp" || $5 == "ack"))\
					print $2, $8 + ($11 % 90) * 0.01
		}
	} out.tr > temp.p
	exec awk {
		{
			if ($1 == "d")
				print $2, $8 + ($11 % 90) * 0.01
		}
	} out.tr > temp.d

	puts $f \"packets
	flush $f
	exec cat temp.p >@ $f
	flush $f
	# insert dummy data sets so we get X's for marks in data-set 4
	puts $f [format "\n\"skip-1\n0 1\n\n\"skip-2\n0 1\n\n"]

	puts $f \"drops
	flush $f
	#
	# Repeat the first line twice in the drops file because
	# often we have only one drop and xgraph won't print marks
	# for data sets with only one point.
	#
	exec head -1 temp.d >@ $f
	exec cat temp.d >@ $f
	close $f
	#exec csh figure.com $file
	exec xgraph -bb -tk -nl -m -x time -y packet temp.rands &
	
	exit 0
}

proc openTrace { stopTime testName } {
	exec rm -f out.tr temp.rands
	global r1 k1    
	set traceFile [open out.tr w]
	ns at $stopTime \
		"close $traceFile ; finish $testName"
	set T [ns trace]
	$T attach $traceFile
	return $T
}

proc test_one_14 {} {
	global s1 s2 r1 k1
	create_testnet

	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 14
	set ftp1 [$tcp1 source ftp]

	ns at 1.0 "$ftp1 start"

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 5.0 test_type_one_14]

	ns run
}

proc test_one_15 {} {
	global s1 s2 r1 k1
	create_testnet

	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 15
	set ftp1 [$tcp1 source ftp]

	ns at 1.0 "$ftp1 start"

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 5.0 test_type_one_15]

	ns run
}

proc test_one_20 {} {
	global s1 s2 r1 k1
	create_testnet

	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 20
	set ftp1 [$tcp1 source ftp]

	ns at 1.0 "$ftp1 start"

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 10.0 test_type_one_20]

	ns run
}

proc test_one_26 {} {
	global s1 s2 r1 k1
	create_testnet

	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 26
	set ftp1 [$tcp1 source ftp]

	ns at 1.0 "$ftp1 start"

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 10.0 test_type_one_26]

	ns run
}

# delayed ack not implemented for sack.
proc test_one_40 {} { 
        global s1 s2 r1 r2 s3 s4
        create_testnet2
        [ns link $r1 $r2] set queue-limit 29
        set tcp1 [ns_create_connection tcp $s1 tcp-sink-da $r2 0]
        $tcp1 set window 40

        set ftp1 [$tcp1 source ftp]
        ns at 0.0 "$ftp1 start"

        # trace only the bottleneck link
        [ns link $r1 $r2] trace [openTrace 5.0 test_type_one_40]

        ns run
}

proc test_two_2 {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $r1 $k1] set queue-limit 8
	[ns link $k1 $r1] set queue-limit 8
	
	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 100

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 16

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 1.0 "$ftp1 start"
	ns at 0.5 "$ftp2 start"

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 8.0 test_type_two_2]

	ns run
}

proc test_two_A {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $r1 $k1] set queue-limit 9

	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 50

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 20

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 1.0 "$ftp1 start"
	ns at 1.0 "$ftp2 start"

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 10.0 test_type_two_A]

	ns run
}

proc test_two_B {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $s2 $r1] set delay 200ms
	[ns link $r1 $s2] set delay 200ms
	[ns link $r1 $k1] set queue-limit 11
	[ns link $k1 $r1] set queue-limit 11
	
	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 30

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 30

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 0.0 "$ftp1 start"
	ns at 0.0 "$ftp2 start"

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 10.0 test_type_two_B]

	ns run
}

if { $argc != 1 } {
	puts stderr {usage: ns test-suite.tcl [ 1 2 3 ... ]}
	exit 1
}
if { "[info procs test_$argv]" != "test_$argv" } {
	puts stderr "sack.tcl: no such test: $argv"
}
test_$argv

